﻿using Microsoft.Extensions.Logging;
using ProjectRepairWork.Repositories;
using ProjectRepairWork.Repositories.Implementation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Reports;

public class ChartReport
{
    private readonly IMaterialProcurementRepository _materialProcurementRepository;
    private readonly IBuildingMaterialsRepository _buildingMaterialsRepository;
    private readonly ILogger<ChartReport> _logger;

    public ChartReport(IMaterialProcurementRepository materialProcurementRepository, IBuildingMaterialsRepository buildingMaterialsRepository, ILogger<ChartReport> logger)
    {
        _buildingMaterialsRepository = buildingMaterialsRepository ?? throw new ArgumentNullException(nameof(buildingMaterialsRepository));
        _materialProcurementRepository = materialProcurementRepository ?? throw new ArgumentNullException(nameof(materialProcurementRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateChart(string filePath, DateTime dateTime)
    {
        try
        {
            new PdfBuilder(filePath)
                .AddHeader("Поступление материалов")
                .AddPieChart("Виды материала", GetData(dateTime))
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<(string Caption, double Value)> GetData(DateTime dateTime)
    {
        var materialProcurement = _materialProcurementRepository.ReadMaterialProcurement()
        .Where(x => x.ProcurmentDate.Date == dateTime.Date).ToList();

        var materials = _buildingMaterialsRepository.ReadBuildingMaterials();


        var data = from materialProc in materialProcurement
                   join material in materials on materialProc.MaterialsId equals material.Id
                   select new
                   {
                       material.MaterialsType,
                       materialProc.MaterialsPrice
                   };


        var groupedData = data
            .GroupBy(x => x.MaterialsType)
            .Select(g => new
            {
                MaterialsName = g.Key,
                TotalCount = g.Sum(x => x.MaterialsPrice)
            })
            .ToList();


        var result = groupedData
        .Select(x => (x.MaterialsName.ToString(), (double)x.TotalCount))
        .ToList();

        return result;
    }
}
