﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectRepairWork.Repositories;
using ProjectRepairWork.Repositories.Implementation;
using ProjectRepairWork.Entities;

namespace ProjectRepairWork.Reports;

internal class TableReport
{ 
    private readonly IRepairRepository _repairRepository;
    private readonly IWorksRepository _worksRepository;
    private readonly ILogger<TableReport> _logger;

    internal static readonly string[] Headers = { "Работа", "Дата", "Количество" };

    public TableReport(IRepairRepository repairRepository, IWorksRepository worksRepository, ILogger<TableReport> logger)
    {
        _repairRepository = repairRepository ?? throw new ArgumentNullException(nameof(repairRepository));
        _worksRepository = worksRepository ?? throw new ArgumentNullException(nameof(worksRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateTable(string filePath, int contractorsId, DateTime startDate, DateTime endDate)
    {
        try
        {
            var data = GetData(contractorsId, startDate, endDate);
            new ExcelBuilder(filePath)
                .AddHeader("Сводка по подрядчикам", 0, 3)
                .AddParagraph("за период", 0)
                .AddTable(new int[] { 10, 15, 15 }, data)
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetData(int contractorsId, DateTime startDate, DateTime endDate)
    {
        var repairs = _repairRepository.ReadRepair()
            .Where(x => x.DateRepair >= startDate && x.DateRepair <= endDate && x.ContractorsId == contractorsId)
            .ToList();

        var repairRepairs = _repairRepository.ReadRepairRepair();
        var works = _worksRepository.ReadWorks();

        var data = from repair in repairs
                   join repairRepair in repairRepairs on repair.Id equals repairRepair.Id
                   join work in works on repairRepair.WorkId equals work.Id
                   select new
                   {
                       work.WorkName,
                       repairRepair.Count,
                       repair.DateRepair
                   };


        var groupedData = data
            .GroupBy(x => new { x.WorkName, x.DateRepair })
            .Select(g => new
            {
                WorksName = g.Key.WorkName,
                Date = g.Key.DateRepair,
                Quantity = g.Sum(x => x.Count)
            })
            .OrderBy(x => x.Date)
            .ToList();

        var totalQuantity = groupedData.Sum(x => x.Quantity);

        var result = new List<string[]>() { Headers }
            .Union(
                groupedData
                    .Select(x => new string[] {
                            x.WorksName.ToString(), x.Date.ToString(), x.Quantity.ToString()
                    }))
            .Union(
                new List<string[]>() { new string[] { "Всего", "", totalQuantity.ToString() } })
            .ToList();

        return result;
    }

}


